/* The 1. Salt -> string containing anything, 25 chars */
#define SALT1 "encdajopmfklibghencdajopm"

/* The 2. Salt -> string containing anything, 25 chars */
#define SALT2 "fklibghencdajopmfklibghen"

/* the 1. Code -> a one byte startup code */
#define CODE1 66

/* the 2. Code -> a one byte startup code */
#define CODE2 67

/* the 1. Salt Offset -> value from 0-24 */
#define SA1 0

/* the 2. Salt Offset -> value from 0-24 */
#define SA2 24

/* the make salt routine */
/* dont wonder about the redundance, its needed to somehow hide the fully salts */

/* salt buffers */

unsigned char slt1[26];
unsigned char slt2[26];

int makesalt(void)
{
    slt1[0]=SALT1[0];
    slt1[1]=SALT1[1];
    slt1[2]=SALT1[2];
    slt1[3]=SALT1[3];
    slt1[4]=SALT1[4];
    slt1[5]=SALT1[5];
    slt1[6]=SALT1[6];
    slt1[7]=SALT1[7];
    slt1[8]=SALT1[8];
    slt1[9]=SALT1[9];
    slt1[10]=SALT1[10];
    slt1[11]=SALT1[11];
    slt1[12]=SALT1[12];
    slt1[13]=SALT1[13];
    slt1[14]=SALT1[14];
    slt1[15]=SALT1[15];
    slt1[16]=SALT1[16];
    slt1[17]=SALT1[17];
    slt1[18]=SALT1[18];
    slt1[19]=SALT1[19];
    slt1[20]=SALT1[20];
    slt1[21]=SALT1[21];
    slt1[22]=SALT1[22];
    slt1[23]=SALT1[23];
    slt1[24]=SALT1[24];
    slt1[25]=0;
    slt2[0]=SALT2[0];
    slt2[1]=SALT2[1];
    slt2[2]=SALT2[2];
    slt2[3]=SALT2[3];
    slt2[4]=SALT2[4];
    slt2[5]=SALT2[5];
    slt2[6]=SALT2[6];
    slt2[7]=SALT2[7];
    slt2[8]=SALT2[8];
    slt2[9]=SALT2[9];
    slt2[10]=SALT2[10];
    slt2[11]=SALT2[11];
    slt2[12]=SALT2[12];
    slt2[13]=SALT2[13];
    slt2[14]=SALT2[14];
    slt2[15]=SALT2[15];
    slt2[16]=SALT2[16];
    slt2[17]=SALT2[17];
    slt2[18]=SALT2[18];
    slt2[19]=SALT2[19];
    slt2[20]=SALT2[20];
    slt2[21]=SALT2[21];
    slt2[22]=SALT2[22];
    slt2[23]=SALT2[23];
    slt2[24]=SALT2[24];
    slt2[25]=0;
}
